import { O as OptionalHasher } from '../shared/gpds.4b85d41b.mjs';
import { CompositeKeyMap } from '../maps/CompositeKeyMap.mjs';
import '../maps/BidirectionalMap.mjs';

class ValuedGraph {
  #storage = new CompositeKeyMap();
  #nodes = /* @__PURE__ */ new Set();
  #directed;
  #nodeHasher;
  #edgeHasher;
  constructor(options = {}) {
    this.#directed = options.directed ?? false;
    this.#nodeHasher = new OptionalHasher(options.hashNode ?? null);
    this.#edgeHasher = new OptionalHasher(options.hashEdge ?? null);
  }
  get directed() {
    return this.#directed;
  }
  add(start, end, edge) {
    const mappedStart = this.#nodeHasher.hash(start);
    const mappedEnd = this.#nodeHasher.hash(end);
    const mappedEdge = this.#edgeHasher.hash(edge);
    this.#nodes.add(mappedStart);
    this.#nodes.add(mappedEnd);
    this.#storage.set([mappedStart, mappedEnd], mappedEdge);
    if (!this.#directed) {
      this.#storage.set([mappedEnd, mappedStart], mappedEdge);
    }
  }
  delete(start, end) {
    const mappedStart = this.#nodeHasher.hash(start);
    const mappedEnd = this.#nodeHasher.hash(end);
    this.#nodes.delete(mappedStart);
    this.#nodes.delete(mappedEnd);
    this.#storage.delete([mappedStart, mappedEnd]);
    if (!this.#directed) {
      this.#storage.delete([mappedEnd, mappedStart]);
    }
  }
  getEdge(start, end) {
    const result = this.#storage.get([
      this.#nodeHasher.hash(start),
      this.#nodeHasher.hash(end)
    ]);
    return result ? this.#edgeHasher.unhash(result) : void 0;
  }
  *nodes() {
    for (const node of this.#nodes) {
      yield this.#nodeHasher.unhash(node);
    }
  }
  *neighborsOf(node) {
    for (const [[neighbor], edge] of this.#storage.entries([
      this.#nodeHasher.hash(node)
    ])) {
      yield {
        node: this.#nodeHasher.unhash(neighbor),
        edge: this.#edgeHasher.unhash(edge)
      };
    }
  }
  // *traverse(
  //   from?: NodeData,
  // ): IterableIterator<{ start: NodeData; end: NodeData; edge: EdgeData }> {
  //   const initial: NodeData | HashResult | undefined =
  //     from !== undefined
  //       ? this.#nodeHasher.hash(from)
  //       : this.#nodes.values().next().value
  //   if (initial === undefined && !this.#nodes.has(undefined)) {
  //     return
  //   }
  //   const visited = new Set<NodeData | HashResult>()
  //   const queue = [initial]
  //   for (const start of queue) {
  //     visited.add(start)
  //     for (const [[end], edge] of this.#storage.entries([start])) {
  //       if (!visited.has(end)) {
  //         queue.push(end)
  //         yield {
  //           start: this.#nodeHasher.unhash(start),
  //           end: this.#nodeHasher.unhash(end),
  //           edge: this.#edgeHasher.unhash(edge),
  //         }
  //       }
  //     }
  //   }
  // }
}

export { ValuedGraph };
