class BidirectionalMap {
  #left = /* @__PURE__ */ new Map();
  #right = /* @__PURE__ */ new Map();
  constructor(iterable) {
    if (iterable) {
      for (const [key, value] of iterable) {
        this.#left.set(key, value);
        this.#right.set(value, key);
      }
    }
  }
  get size() {
    return this.#left.size;
  }
  clear() {
    this.#left.clear();
    this.#right.clear();
  }
  deleteLeft(key) {
    const value = this.#left.get(key);
    if (value !== void 0) {
      this.#left.delete(key);
      this.#right.delete(value);
      return true;
    }
    return false;
  }
  deleteRight(value) {
    const key = this.#right.get(value);
    if (key !== void 0) {
      this.#right.delete(value);
      this.#left.delete(key);
      return true;
    }
    return false;
  }
  getLeft(key) {
    return this.#left.get(key);
  }
  getRight(value) {
    return this.#right.get(value);
  }
  hasLeft(key) {
    return this.#left.has(key);
  }
  hasRight(value) {
    return this.#right.has(value);
  }
  set(key, value) {
    this.#left.set(key, value);
    this.#right.set(value, key);
    return this;
  }
  [Symbol.iterator]() {
    return this.#left[Symbol.iterator]();
  }
  entries() {
    return this.#left.entries();
  }
  keys() {
    return this.#left.keys();
  }
  values() {
    return this.#left.values();
  }
  get [Symbol.toStringTag]() {
    return "BidirectionalMap";
  }
}

export { BidirectionalMap };
