import { O as OptionalHasher } from '../shared/gpds.4b85d41b.mjs';
import './BidirectionalMap.mjs';

class CompositeKeyMap {
  #hasher;
  #compareKeys;
  #storage = [void 0, /* @__PURE__ */ new Map()];
  constructor(options = {}) {
    this.#hasher = new OptionalHasher(options.hashKey);
    this.#compareKeys = options.compareKeys ?? null;
  }
  #traverse(keys, create) {
    let result = this.#storage;
    const chain = this.#compareKeys ? keys.toSorted(this.#compareKeys) : keys;
    for (let i = 0; i < chain.length; i++) {
      const key = this.#hasher.hash(chain[i]);
      const map = result[1];
      let next = map.get(key);
      if (!next) {
        if (create === false) {
          return void 0;
        }
        next = [void 0, /* @__PURE__ */ new Map()];
        map.set(key, next);
      }
      result = next;
    }
    return result;
  }
  set(keys, value) {
    const storage = this.#traverse(keys, true);
    storage[0] = value;
  }
  get(keys) {
    const storage = this.#traverse(keys, false);
    return storage?.[0];
  }
  delete(keys) {
    const storage = this.#traverse(keys, false);
    if (!storage) {
      return false;
    }
    if (storage[0] === void 0) {
      return false;
    }
    storage[0] = void 0;
    return true;
  }
  has(keys) {
    const storage = this.#traverse(keys, false);
    return storage ? storage[0] !== void 0 : false;
  }
  clear() {
    this.#hasher.clear();
    this.#storage = [void 0, /* @__PURE__ */ new Map()];
  }
  *keys(from = []) {
    const storage = this.#traverse(from, false);
    if (!storage) {
      return;
    }
    const stack = [[storage, []]];
    while (stack.length > 0) {
      const [storage2, keys] = stack.shift();
      for (const [key, next] of storage2[1]) {
        if (next[0] !== void 0) {
          yield keys.concat(key).map((key2) => this.#hasher.unhash(key2));
        }
        stack.push([next, keys.concat(key)]);
      }
    }
  }
  *values(from = []) {
    const storage = this.#traverse(from, false);
    if (!storage) {
      return;
    }
    const stack = [storage];
    while (stack.length > 0) {
      const storage2 = stack.shift();
      if (storage2[0] !== void 0) {
        yield storage2[0];
      }
      for (const [, next] of storage2[1]) {
        stack.push(next);
      }
    }
  }
  *entries(from = []) {
    const storage = this.#traverse(from, false);
    if (!storage) {
      return;
    }
    const stack = [[storage, []]];
    while (stack.length > 0) {
      const [storage2, keys] = stack.shift();
      if (storage2[0] !== void 0) {
        yield [keys.map((key) => this.#hasher.unhash(key)), storage2[0]];
      }
      for (const [key, next] of storage2[1]) {
        stack.push([next, keys.concat(key)]);
      }
    }
  }
  *roots() {
    for (const [key] of this.#storage[1]) {
      yield this.#hasher.unhash(key);
    }
  }
}

export { CompositeKeyMap };
