import { O as OptionalHasher } from '../shared/gpds.4b85d41b.mjs';
import './BidirectionalMap.mjs';

class MultiMap {
  #storage = /* @__PURE__ */ new Map();
  #emptySet = /* @__PURE__ */ new Set();
  #hasher;
  constructor(iterable, options = {}) {
    this.#hasher = new OptionalHasher(options.hashKey ?? null);
    if (iterable) {
      for (const [key, value] of iterable) {
        this.set(key, value);
      }
    }
  }
  get size() {
    return this.#storage.size;
  }
  clear() {
    this.#hasher.clear();
    this.#storage.clear();
  }
  delete(key, value) {
    const mapped = this.#hasher.hash(key);
    let existed = false;
    if (value !== void 0) {
      const storage = this.#storage.get(mapped);
      if (storage) {
        existed = storage.delete(value);
      }
    } else {
      existed = this.#storage.delete(mapped);
    }
    return existed;
  }
  get(key) {
    return this.#storage.get(this.#hasher.hash(key)) ?? this.#emptySet;
  }
  has(key) {
    return this.#storage.has(this.#hasher.hash(key));
  }
  set(key, value) {
    const mapped = this.#hasher.hash(key);
    let storage = this.#storage.get(mapped);
    if (!storage) {
      storage = /* @__PURE__ */ new Set();
      this.#storage.set(mapped, storage);
    }
    storage.add(value);
    return this;
  }
  *entries() {
    for (const [key, values] of this.#storage.entries()) {
      yield [this.#hasher.unhash(key), values];
    }
  }
  *keys() {
    for (const key of this.#storage.keys()) {
      yield this.#hasher.unhash(key);
    }
  }
  values() {
    return this.#storage.values();
  }
  [Symbol.toStringTag] = "MultiMap";
}

export { MultiMap };
