import { BidirectionalMap } from '../maps/BidirectionalMap.mjs';

class OptionalHasher {
  #storageMap = null;
  #hasher;
  constructor(mapper = null) {
    this.#hasher = mapper;
  }
  get #storage() {
    if (!this.#storageMap) {
      this.#storageMap = new BidirectionalMap();
    }
    return this.#storageMap;
  }
  hash(input) {
    if (this.#hasher) {
      let mapped = this.#storage.getLeft(input);
      if (!mapped) {
        mapped = this.#hasher(input);
        this.#storage.set(input, mapped);
      }
      return mapped;
    }
    return input;
  }
  unhash(output) {
    if (this.#hasher) {
      return this.#storage.getRight(output);
    }
    return output;
  }
  clear() {
    this.#storage.clear();
  }
}

export { OptionalHasher as O };
